@@ -30,8 +30,6 @@
 #include <kfileutils.h>
 #include <qmimedatabase.h>
 #include <QUrl>
-#include <QLocale>
-#include <QFileInfo>
 
 #include "kiocoredebug.h"
 
@@ -98,7 +96,7 @@ KIOCORE_EXPORT QString KIO::convertSeconds(unsigned int seconds)
     }
 }
 
-#ifndef KIOCORE_NO_DEPRECATED
+#if KIOCORE_BUILD_DEPRECATED_SINCE(3, 4)
 KIOCORE_EXPORT QTime KIO::calculateRemaining(KIO::filesize_t totalSize, KIO::filesize_t processedSize, KIO::filesize_t speed)
 {
     QTime remainingTime;
@@ -227,25 +225,23 @@ QString KIO::favIconForUrl(const QUrl &url)
 
 QString KIO::iconNameForUrl(const QUrl &url)
 {
-    const QLatin1String unknown("unknown");
     if (url.scheme().isEmpty()) { // empty URL or relative URL (e.g. '~')
-        return unknown;
+        return QStringLiteral("unknown");
     }
     QMimeDatabase db;
     const QMimeType mt = db.mimeTypeForUrl(url);
-    const QString mimeTypeIcon = mt.iconName();
-    QString i = mimeTypeIcon;
+    QString iconName;
 
     if (url.isLocalFile()) {
         // Check to see whether it's an xdg location (e.g. Pictures folder)
         if (mt.inherits(QStringLiteral("inode/directory"))) {
-            i = KIOPrivate::iconForStandardPath(url.toLocalFile());
+            iconName = KIOPrivate::iconForStandardPath(url.toLocalFile());
         }
 
         // Let KFileItem::iconName handle things for us
-        if (i == unknown || i.isEmpty() || mt.isDefault()) {
+        if (iconName.isEmpty()) {
             const KFileItem item(url, mt.name());
-            i = item.iconName();
+            iconName = item.iconName();
         }
 
     } else {
@@ -253,29 +249,28 @@ QString KIO::iconNameForUrl(const QUrl &url)
 
         // Look for a favicon
         if (url.scheme().startsWith(QLatin1String("http"))) {
-            i = favIconForUrl(url);
+            iconName = favIconForUrl(url);
         }
 
         // Then handle the trash
-        else if (url.scheme() == QLatin1String("trash") && url.path().length() <= 1) {
-            KConfig trashConfig(QStringLiteral("trashrc"), KConfig::SimpleConfig);
-            if (trashConfig.group("Status").readEntry("Empty", true)) {
-                i = QStringLiteral("user-trash");
-            } else {
-                i = QStringLiteral("user-trash-full");
+        else if (url.scheme() == QLatin1String("trash")) {
+            if (url.path().length() <= 1) { // trash:/ itself
+                KConfig trashConfig(QStringLiteral("trashrc"), KConfig::SimpleConfig);
+                iconName = trashConfig.group("Status").readEntry("Empty", true) ?
+                           QStringLiteral("user-trash") : QStringLiteral("user-trash-full");
+            } else { // url.path().length() > 1, it's a file/folder under trash:/
+                iconName = mt.iconName();
             }
         }
 
-        if (i.isEmpty()) {
-            i = KProtocolInfo::icon(url.scheme());
-        }
-
-        // root of protocol: if we found nothing, revert to mimeTypeIcon (which is usually "folder")
-        if (url.path().length() <= 1 && (i == unknown || i.isEmpty())) {
-            i = mimeTypeIcon;
+        // and other protocols
+        if (iconName.isEmpty()) {
+            iconName = KProtocolInfo::icon(url.scheme());
         }
     }
-    return !i.isEmpty() ? i : unknown;
+    // if we found nothing, return QMimeType.iconName()
+    // (which fallbacks to "application-octet-stream" when no mimetype could be determined)
+    return !iconName.isEmpty() ? iconName : mt.iconName();
 }
 
 QUrl KIO::upUrl(const QUrl &url)
@@ -296,7 +291,7 @@ QUrl KIO::upUrl(const QUrl &url)
     return u.adjusted(QUrl::RemoveFilename);
 }
 
-#ifndef KIOCORE_NO_DEPRECATED
+#if KIOCORE_BUILD_DEPRECATED_SINCE(5, 61)
 QString KIO::suggestName(const QUrl &baseURL, const QString &oldName)
 {
     return KFileUtils::suggestName(baseURL, oldName);
