@@ -162,52 +162,28 @@ static void setProxyInformation(const QString& value,
     }
 
     if (spinBox) {
-        QString urlStr;
-        int portNum = -1;
-        int index = value.lastIndexOf(QL1C(' '));
-
-        if (index > 0) {
-            bool ok = false;
-            portNum = value.midRef(index+1).toInt(&ok);
-            if (!ok) {
-                portNum = -1;
-            }
-            urlStr = value.left(index).trimmed();
-        } else {
-            urlStr = value.trimmed();
-        }
-
         KUriFilterData data;
-        data.setData(urlStr);
+        data.setData(value);
         data.setCheckForExecutables(false);
         if (!defaultScheme.isEmpty()) {
             data.setDefaultUrlScheme(defaultScheme);
         }
 
+        QUrl url;
         if (KUriFilter::self()->filterUri(data, QStringList() << QStringLiteral("kshorturifilter"))) {
-            QUrl url (data.uri());
-            if (portNum == -1 && url.port() > -1) {
-                portNum = url.port();
-            }
-
-            url.setPort(-1);
+            url = QUrl(data.uri());
             url.setUserName(QString());
             url.setPassword(QString());
             url.setPath(QString());
-
-            manEdit->setText(((KSaveIOConfig::proxyDisplayUrlFlags() & flag) ? url.host(): url.url()));
         } else {
-            QUrl url(urlStr);
-            if (portNum == -1 && url.port() > -1) {
-                portNum = url.port();
-            }
-            url.setPort(-1);
-            manEdit->setText((KSaveIOConfig::proxyDisplayUrlFlags() & flag) ? url.host() : urlStr);
+            url = QUrl(value);
         }
 
-        if (spinBox && portNum > -1) {
-            spinBox->setValue(portNum);
+        if (url.port() > -1) {
+            spinBox->setValue(url.port());
         }
+        url.setPort(-1);
+        manEdit->setText((KSaveIOConfig::proxyDisplayUrlFlags() & flag) ? url.host() : url.url());
         return;
     }
 
@@ -428,8 +404,12 @@ void KProxyDialog::defaults()
 
 bool KProxyDialog::autoDetectSystemProxy(QLineEdit* edit, const QString& envVarStr, bool showValue)
 {
+#if QT_VERSION < QT_VERSION_CHECK(5, 15, 0)
     const QStringList envVars = envVarStr.split(QL1C(','), QString::SkipEmptyParts);
-    Q_FOREACH (const QString & envVar, envVars) {
+#else
+    const QStringList envVars = envVarStr.split(QL1C(','), Qt::SkipEmptyParts);
+#endif
+    for (const QString & envVar : envVars) {
         const QByteArray envVarUtf8(envVar.toUtf8());
         const QByteArray envVarValue = qgetenv(envVarUtf8.constData());
         if (!envVarValue.isEmpty()) {
