@@ -35,22 +35,22 @@
 #include <kjobtrackerinterface.h>
 #include <kio/jobuidelegate.h>
 #include <job_p.h>
+#include "kio_widgets_debug.h"
 
 #include <QDateTime>
 #include <QDBusConnection>
 #include <QFileInfo>
+#include <QLocale>
 
 #include <assert.h>
 
 using namespace KIO;
 
-#if 0
 static const char *undoStateToString(UndoState state)
 {
     static const char *const s_undoStateToString[] = { "MAKINGDIRS", "MOVINGFILES", "STATINGFILE", "REMOVINGDIRS", "REMOVINGLINKS" };
     return s_undoStateToString[state];
 }
-#endif
 
 static QDataStream &operator<<(QDataStream &stream, const KIO::BasicOperation &op)
 {
@@ -83,6 +83,16 @@ static QDataStream &operator>>(QDataStream &stream, UndoCommand &cmd)
     return stream;
 }
 
+QDebug operator<<(QDebug dbg, const BasicOperation &op)
+{
+    if (op.m_valid) {
+        static const char* s_types[] = { "File", "Link", "Directory" };
+        dbg << "BasicOperation: type" << s_types[op.m_type] << "src" << op.m_src << "dest" << op.m_dst << "target" << op.m_target << "renamed" << op.m_renamed;
+    } else {
+        dbg << "Invalid BasicOperation";
+    }
+    return dbg;
+}
 /**
  * checklist:
  * copy dir -> overwrite -> works
@@ -121,7 +131,7 @@ public:
         d_ptr->m_caption = i18n("Undo Changes");
         d_ptr->m_message = i18n("Undoing this operation requires root privileges. Do you want to continue?");
     }
-    virtual ~UndoJob() {}
+    ~UndoJob() override {}
 
     virtual void kill(bool)
     {
@@ -182,6 +192,7 @@ CommandRecorder::~CommandRecorder()
 void CommandRecorder::slotResult(KJob *job)
 {
     if (job->error()) {
+        qWarning() << job->errorString();
         return;
     }
 
@@ -197,6 +208,7 @@ void CommandRecorder::slotCopyingDone(KIO::Job *, const QUrl &from, const QUrl &
     op.m_src = from;
     op.m_dst = to;
     op.m_mtime = mtime;
+    //qDebug() << op;
     m_cmd.m_opStack.prepend(op);
 }
 
@@ -369,6 +381,8 @@ quint64 FileUndoManager::currentCommandSerialNumber() const
 
 void FileUndoManager::undo()
 {
+    Q_ASSERT(!d->m_commands.isEmpty()); // forgot to record before calling undo?
+
     // Make a copy of the command to undo before slotPop() pops it.
     UndoCommand cmd = d->m_commands.last();
     assert(cmd.m_valid);
@@ -449,7 +463,7 @@ void FileUndoManager::undo()
         d->m_fileCleanupStack.append(d->m_current.m_dst);
     }
 
-    //qDebug() << "starting with" << undoStateToString(d->m_undoState);
+    qCDebug(KIO_WIDGETS) << "starting with" << undoStateToString(d->m_undoState);
     d->m_undoJob = new UndoJob(d->m_uiInterface->showProgressInfo());
     QMetaObject::invokeMethod(d, "undoStep", Qt::QueuedConnection);
 }
@@ -477,6 +491,7 @@ void FileUndoManagerPrivate::slotResult(KJob *job)
 {
     m_currentJob = nullptr;
     if (job->error()) {
+        qWarning() << job->errorString();
         m_uiInterface->jobError(static_cast<KIO::Job *>(job));
         delete m_undoJob;
         stopUndo(false);
@@ -486,7 +501,7 @@ void FileUndoManagerPrivate::slotResult(KJob *job)
         KIO::StatJob *statJob = static_cast<KIO::StatJob *>(job);
         const QDateTime mtime = QDateTime::fromMSecsSinceEpoch(1000 * statJob->statResult().numberValue(KIO::UDSEntry::UDS_MODIFICATION_TIME, -1), Qt::UTC);
         if (mtime != op.m_mtime) {
-            //qDebug() << op.m_dst << " was modified after being copied!";
+            qCDebug(KIO_WIDGETS) << op.m_dst << "was modified after being copied. Initial timestamp" << mtime << "now" << op.m_mtime;
             QDateTime srcTime = op.m_mtime.toLocalTime();
             QDateTime destTime = mtime.toLocalTime();
             if (!m_uiInterface->copiedFileWasModified(op.m_src, op.m_dst, srcTime, destTime)) {
@@ -740,7 +755,7 @@ bool FileUndoManager::UiInterface::copiedFileWasModified(const QUrl &src, const
 {
     Q_UNUSED(srcTime); // not sure it should appear in the msgbox
     // Possible improvement: only show the time if date is today
-    const QString timeStr = destTime.toString(Qt::DefaultLocaleShortDate);
+    const QString timeStr = QLocale().toString(destTime, QLocale::ShortFormat);
     return KMessageBox::warningContinueCancel(
                d->m_parentWidget,
                i18n("The file %1 was copied from %2, but since then it has apparently been modified at %3.\n"
